/*
 * Copyright (c) 2024.
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.antragsraum.security;

import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;

import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.web.authentication.logout.SecurityContextLogoutHandler;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import lombok.RequiredArgsConstructor;

@RestController
@RequiredArgsConstructor
@RequestMapping(AuthenticationController.PATH)
class AuthenticationController {
	static final String PATH = "/auth"; // NOSONAR

	private final InMemoryUserDetailService userDetailsService;
	private final JwtTokenProvider tokenProvider;
	private final SecurityContextLogoutHandler logoutHandler;

	@PostMapping(path = "/authenticate", produces = MediaType.TEXT_PLAIN_VALUE, consumes = MediaType.APPLICATION_JSON_VALUE)
	ResponseEntity<String> login(@RequestBody AuthCode data) {
		String token;
		try {
			token = createJwt(data.code());
		} catch (Exception e) {
			throw new SecurityException("Error creating jwt: ", e);
		}
		return ResponseEntity.ok().body(token);
	}

	private String createJwt(String code) {
		return tokenProvider.generate(userDetailsService.getUserByCode(code));
	}

	@PostMapping("/refresh")
	ResponseEntity<String> refresh(@RequestBody AuthCode code) {
		var userOptional = userDetailsService.getUser(code);

		return userOptional.map(this::tokenRefreshResponse).orElse(ResponseEntity.status(HttpStatus.FORBIDDEN).build());
	}

	ResponseEntity<String> tokenRefreshResponse(User user) {
		return ResponseEntity.ok().body(tokenProvider.generate(userDetailsService.updateRefreshCodeOf(user)));
	}

	@GetMapping(path = "/logout")
	ResponseEntity<String> logout(HttpServletRequest request, HttpServletResponse response, @AuthenticationPrincipal User user) {
		if (user != null) {
			userDetailsService.logout(user);
		}

		var authentication = SecurityContextHolder.getContext().getAuthentication();
		logoutHandler.logout(request, response, authentication);

		return ResponseEntity.ok("Success");
	}
}
