/*
 * Copyright (c) 2024.   Das Land Schleswig-Holstein vertreten durch den Ministerpräsidenten
 * des Landes Schleswig-Holstein Staatskanzlei Abteilung Digitalisierung und zentrales IT-Management der Landesregierung.
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.antragsraum.attachments;

import static de.ozgcloud.antragsraum.attachments.FileRestClient.*;
import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;
import static org.springframework.test.web.client.match.MockRestRequestMatchers.*;
import static org.springframework.test.web.client.response.MockRestResponseCreators.*;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.UUID;
import java.util.concurrent.CompletableFuture;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.client.RestClientTest;
import org.springframework.boot.test.web.client.MockServerRestClientCustomizer;
import org.springframework.http.MediaType;
import org.springframework.test.context.junit.jupiter.SpringJUnitConfig;
import org.springframework.test.web.client.MockRestServiceServer;
import org.springframework.web.client.RestClient;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;

import de.ozgcloud.antragsraum.common.TechnicalException;
import de.ozgcloud.antragsraum.proxy.AntragraumproxyGrpcFindFilesResponse;

@SpringJUnitConfig(classes = { FileRestClientTestConfiguration.class })
@RestClientTest(FileRestClient.class)
class FileRestClientTest {
	final static String ADDRESS = "http://localhost:8382";
	public static final String FILE_ID = "test";
	private FileRestClient fileRestClient;

	@Autowired
	private RestClient.Builder restClientBuilder;

	@Autowired
	private MockServerRestClientCustomizer customizer;

	private MockRestServiceServer server;

	private final ObjectMapper objectMapper = new ObjectMapper();

	@BeforeEach
	void setupRestClient() {
		customizer.customize(restClientBuilder);
		server = customizer.getServer(restClientBuilder);
		fileRestClient = new FileRestClient(restClientBuilder.build());
	}

	@Nested
	class TestLoadingFileMetadata {
		private final String fileFileUri = ADDRESS + FIND_FILE_METADATA_API.replace("{fileId}", FILE_ID);

		@BeforeEach
		void setup() throws JsonProcessingException {
			server.expect(requestTo(fileFileUri))
			  .andRespond(withSuccess(objectMapper.writeValueAsString(createFindFileMetadataResponse()), MediaType.APPLICATION_JSON));
		}

		private AntragraumproxyGrpcFindFilesResponse createFindFileMetadataResponse() {
			return AntragraumproxyGrpcFindFilesResponseTestFactory.create();
		}

		@Test
		void shouldFindFileMetadata() {
			var fileMetadata = fileRestClient.findBinaryFilesMetaData(FILE_ID, ADDRESS + FIND_FILE_METADATA_API);

			assertThat(fileMetadata).isNotNull();
		}
	}

	@Nested
	class TestUploadingFile {

		@Test
		void shouldUploadFile() throws JsonProcessingException {
			server.expect(requestTo(ADDRESS + UPLOAD_FILE_API))
			  .andRespond(withSuccess(objectMapper.writeValueAsString(UUID.randomUUID().toString()), MediaType.APPLICATION_JSON));

			CompletableFuture<String> fileId = new CompletableFuture<>();
			fileRestClient.uploadFile(OzgUploadFileTestFactory.create(), ADDRESS + UPLOAD_FILE_API, fileId);

			assertThat(fileId).isCompleted();
		}

		@Test
		void shouldHandleEmptyUploadFile() {
			server.expect(requestTo(ADDRESS + UPLOAD_FILE_API))
			  .andRespond(withSuccess("", MediaType.APPLICATION_JSON));

			CompletableFuture<String> fileId = new CompletableFuture<>();
			fileRestClient.uploadFile(OzgUploadFileTestFactory.create(), ADDRESS + UPLOAD_FILE_API, fileId);

			assertThat(fileId).withFailMessage(NO_FILE_ID_RECEIVED_MESSAGE).isCompletedExceptionally();
		}
	}

	@Nested
	class TestDownloadFileContent {
		private final String fileContentUri = ADDRESS + GET_FILE_CONTENT_API.replace("{fileId}", FILE_ID);

		@BeforeEach
		void setup() throws IOException {
			server.expect(requestTo(fileContentUri))
			  .andRespond(withSuccess(OzgUploadFileTestFactory.MULTIPART_FILE.getBytes(), MediaType.APPLICATION_OCTET_STREAM));
		}

		@Test
		void shouldDownloadFileContent() throws IOException {
			var out = new ByteArrayOutputStream();
			fileRestClient.downloadFileContent(FILE_ID, out, ADDRESS + GET_FILE_CONTENT_API);
			var res = out.toByteArray();

			assertThat(res).isEqualTo(OzgUploadFileTestFactory.MULTIPART_FILE.getBytes());
		}

		@Test
		void shouldDownloadFileContentIoException() {
			var out = mock(ByteArrayOutputStream.class);
			doThrow(IOException.class).when(out).write(any(byte[].class), anyInt(), anyInt());

			assertThatExceptionOfType(TechnicalException.class).isThrownBy(
			  () -> fileRestClient.downloadFileContent(FILE_ID, out, ADDRESS + GET_FILE_CONTENT_API));
		}
	}
}