/*
 * Copyright (c) 2024.
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.antragsraum.security;

import static org.mockito.Mockito.*;

import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.security.saml2.provider.service.authentication.DefaultSaml2AuthenticatedPrincipal;
import org.springframework.security.saml2.provider.service.authentication.Saml2Authentication;

@ExtendWith(MockitoExtension.class)
class SamlRedirectStrategyTest {
	private static final String URL = "http://test";

	@Mock
	InMemoryUserDetailService userDetailsService;

	@Mock
	HttpServletRequest request;

	@Mock
	HttpServletResponse response;

	@Mock
	private Saml2Authentication authentication;

	@Mock
	User user;

	SamlRedirectStrategy samlRedirectStrategy;

	@Nested
	class TestRedirect {
		@BeforeEach
		void setup() {
			when(request.getContextPath()).thenReturn(URL);
			when(response.encodeRedirectURL(matches(URL + "\\?code=*"))).thenReturn(URL + "?code=abc");
			samlRedirectStrategy = new SamlRedirectStrategy(userDetailsService, null);
		}

		@Test
		void shouldAddUserToUserDetailservice() throws IOException {
			initAuthentication();

			samlRedirectStrategy.sendRedirect(request, response, URL, authentication);

			verify(userDetailsService).addUser(anyString(), any(User.class));
		}

		private void initAuthentication() {
			Map<String, List<Object>> attributes = new HashMap<>();
			attributes.put(UserMapper.NACHNAME_URN, List.of(UserTestFactory.LAST_NAME));
			attributes.put(UserMapper.VORNAME_URN, List.of(UserTestFactory.FIRST_NAME));
			attributes.put(UserMapper.POSTKORB_HANDLE_URN, List.of(UserTestFactory.POSTKORB_HANDLE));
			attributes.put(UserMapper.VERTRAUENSNIVEAU_URN, List.of(UserTestFactory.TRUST_LEVEL));
			attributes.put(UserMapper.BK2_URN, List.of(UserTestFactory.USER_NAME));

			var principal = new DefaultSaml2AuthenticatedPrincipal(UserTestFactory.USER_ID, attributes);
			when(authentication.getPrincipal()).thenReturn(principal);
		}

		@Test
		void shouldCreateRedirectUrl() throws IOException {
			samlRedirectStrategy.sendRedirect(request, response, URL);

			verify(response).sendRedirect(matches(URL + "\\?code=*"));
		}
	}

	@Nested
	class TestRedirectWithConfiguredUrl {
		private final String CONFIGURED_REDIRECT_URL = "https://redirect.me";

		@BeforeEach
		void setup() {
			when(response.encodeRedirectURL(matches(CONFIGURED_REDIRECT_URL + "\\?code=*"))).thenReturn(CONFIGURED_REDIRECT_URL + "?code=abc");
			samlRedirectStrategy = new SamlRedirectStrategy(userDetailsService, CONFIGURED_REDIRECT_URL);
		}

		@Test
		void shouldCreateRedirectUrl() throws IOException {
			samlRedirectStrategy.sendRedirect(request, response, URL);

			verify(response).sendRedirect(CONFIGURED_REDIRECT_URL + "?code=abc");
		}
	}
}